;  Copyright (C) 2006 Red Hat Inc.
;
;  This file is part of GAS, the GNU Assembler.
;
;  GAS is free software; you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation; either version 2, or (at your option)
;  any later version.
;
;  GAS is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with GAS; see the file COPYING.  If not, write to
;  the Free Software Foundation, 59 Temple Place - Suite 330,
;  Boston, MA 02111-1307, USA.  */

macro	sdsp8	{-128;-1;0;1;127;0x7f}
macro	udsp8	{0;1;255;0xff}
macro	sdsp16	{-32768;-1;0;1;32767;0x7fff;sym}
macro	udsp16	{0;1;65535;0xffff;sym}
macro	abs16	{0;1;65535;0xffff;sym}
macro	abs16s	{0;1;65535;0xffff;%dsp16(sym)}
macro	udsp24	{0;1;16777215;0xffffff;sym}
macro	abs24	{0;1;16777215;0xffffff;sym}

macro	daddrs	{{{udsp8};{udsp16};{udsp24}}[a{0;1}];{{udsp8};{udsp16}}[sb];{{sdsp8};{sdsp16}}[fb]}
macro	iaddrs	{{;{udsp8};{udsp16};{udsp24}}[a{0;1}];{{udsp8};{udsp16}}[sb];{{sdsp8};{sdsp16}}[fb]}
macro	addrs	{{iaddrs};{abs16};{abs24}}
macro	fb8	{sdsp8}[fb]
macro	sb8	{udsp8}[sb]
macro	sp8	{sdsp8}[sp]

macro	uimm3	#{0;7}
macro	pimm3	#{1;8}
macro	simm4	#{-8;0;7}
macro	simm4i	#{-7;0;8}
macro	nimm4	#{-8;-1;1;8}
macro	uimm8	#{udsp8}
macro	simm8	#{sdsp8}
macro	dimm8	#{0;1;99}
macro	imm8	#{-127;-1;{udsp8}}
macro	uimm16	#{udsp16}
macro	simm16	#{sdsp16}
macro	dimm16	#{0;1;9999}
macro	imm16	#{-32768;-1;{udsp16}}
macro	uimm24	#{udsp24}
macro	imm24	#{-8388608;-1;{udsp24}}
macro	uimm32	#{0;1;4294967295;0xffffffff}
macro	simm32	#{-2147483648;-1;0;1;2147483647;0x7fffffff}
macro	imm32	#{-2147483648;-1;0;1;4294967295;0xffffffff}

macro	sdb	{r0l;r0h;r1l;r1h;a0;a1;{addrs}}
macro	sdw	{r0;r1;r2;r3;a0;a1;{addrs}}
macro	sdl	{r2r0;r3r1;a0;a1;{addrs}}

macro	isdb	{{sdb};[{iaddrs}]}
macro	isdw	{{sdw};[{iaddrs}]}
macro	isdl	{{sdl};[{iaddrs}]}

macro	addr2	{{udsp8}[sb];{sdsp8}[fb];{abs16s}}
macro	iaddr2	{[{addr2}];{addr2}}
macro	dst2b	{r0l;{addr2}}
macro	dst2w	{r0;{addr2}}
macro	idst2b	{[{addr2}];{dst2b}}
macro	idst2w	{[{addr2}];{dst2w}}

macro	cr16	{dct0;dct1;flg;svf;drc0;drc1;dmd0;dmd1}
macro	cr24	{intb;sp;sb;fb;svp;vct;isp}
macro	cr24a	{{cr24};dma0;dma1;dra0;dra1;dsa0;dsa1}
macro	flg	{u;i;o;b;s;z;d;c}

macro	sdsp5	{-16;-1;0;1;15;0x1f}
macro	udsp5	{0;1;31;0x1f}
macro	udsp13	{0;1;8191;0x1fff;sym}
macro	bit3	{0;7}
macro	bit4	{0;15}

macro	bsrc	{bit3},{sdb}

macro	cond	{ltu;nc;leu;ne;nz;pz;no;gt;ge;geu;c;gtu;eq;z;n;o;le;lt}
macro	label3	{.+2;.+9}
macro	label8	{.-126;.+128;foo}
macro	label16	{.+2;foo}
macro	label24	{.+2;0x100000;foo}

macro	g	{;:g}

	abs.b		{isdb}

	adc.b		{imm8},{sdb}
	adc.w		{imm16},{sdw}
	adc.b		{sdb},{sdb}
	adc.w		{sdw},{sdw}

	adcf.b		{sdb}
	adcf.w		{sdw}

	add.b{g}	{imm8},{isdb}
	add.w{g}	{imm16},{isdw}
	add.l{g}	{imm32},{isdl}
	add.b:q		{simm4},{isdb}
	add.w:q		{simm4},{isdw}
	add.b:s		{imm8},{idst2b}
	add.l:s		#{1;2},{a0;a1}
	add.b{g}	{isdb},{isdb}
	add.w{g}	{isdw},{isdw}
	add.l{g}	{isdl},{isdl}
	add.l{g}	{simm16},sp
	add.l:q		{pimm3},sp
	add.l:s		{simm8},sp

	addx		{simm8},{isdl}
	addx		{sdb},{sdl}

	adjnz.b		{simm4},{sdb},{label8}
	adjnz.w		{simm4},{sdw},{label8}

	and.b{g}	{imm8},{isdb}
	and.w{g}	{imm16},{isdw}
	and.b:s		{imm8},{idst2b}
	and.w:s		{imm8},{idst2w}
	and.b{g}	{isdb},{isdb}
	and.w{g}	{isdw},{isdw}

	band		{bsrc}
	bclr		{bsrc}

	bitindex.b	{sdb}
	bitindex.w	{sdw}

	bm{cond}	{bsrc}
	bm{cond}	c

	bnand		{bsrc}
	bnor		{bsrc}
	bnot		{bsrc}
	bntst		{bsrc}
	bnxor		{bsrc}
	bor		{bsrc}

	brk
	brk2

	bset		{bsrc}

	btst{g}		{bsrc}
	btst:s		{bit3},{abs16s}

	btstc		{bsrc}
	btsts		{bsrc}
	bxor		{bsrc}

	clip.b		{simm8},{simm8},{sdb}
	clip.w		{simm16},{simm16},{sdw}

	cmp.b{g}	{imm8},{isdb}
	cmp.w{g}	{imm16},{isdw}
	cmp.l{g}	{imm32},{isdl}
	cmp.b:q		{simm4},{isdb}
	cmp.w:q		{simm4},{isdw}
	cmp.b:s		{imm8},{idst2b}
	cmp.w:s		{imm16},{idst2w}
	cmp.b{g}	{isdb},{isdb}
	cmp.w{g}	{isdw},{isdw}
	cmp.l{g}	{isdl},{isdl}
	cmp.b:s		{iaddr2},r0l
	cmp.w:s		{iaddr2},r0

	cmpx		{simm8},{isdl}

	d{adc;add}.b	{{dimm8};{sdb}},{sdb}
	d{adc;add}.w	{{dimm16};{sdw}},{sdw}

	dec.b		{isdb}
	dec.w		{isdw}

	div.b		{simm8}
	div.w		{simm16}
	div.b		{isdb}
	div.w		{isdw}

	divu.b		{uimm8}
	divu.w		{uimm16}
	divu.b		{isdb}
	divu.w		{isdw}

	divx.b		{simm8}
	divx.w		{simm16}
	divx.b		{isdb}
	divx.w		{isdw}

	d{sbb;sub}.b	{{dimm8};{sdb}},{sdb}
	d{sbb;sub}.w	{{dimm16};{sdw}},{sdw}

	enter		{uimm8}
	exitd

	exts.b		{r0l;r1l;{addrs}}
	exts.w		{r0;r1;{addrs}}
	exts.b		{r0l;r1l;r0h;r1h;{addrs}},{r0;r1;r2;r3;{addrs}}
	extz		{r0l;r1l;r0h;r1h;{addrs}},{r0;r1;r2;r3;{addrs}}

	fclr		{flg}

	freit

	fset		{flg}

	inc.b		{isdb}
	inc.w		{isdw}

	index{b;w;l}{;d;s}.b	{sdb}
	index{b;w;l}{;d;s}.w	{sdw}

	int		#{0;63}
	into

	j{cond}		{label8}

	jmp.s		{label3}
	jmp.b		{label8}
	jmp.w		{label16}
	jmp.a		{label24}

	jmpi.w		{sdw}
	jmpi.a		{sdl}

	jmps		#{18;255}

	jsr.w		{label16}
	jsr.a		{label24}

	jsri.w		{sdw}
	jsri.a		{sdl}

	jsrs		#{18;255}

	ldc		{imm16},{cr16}
	ldc		{uimm24},{cr24a}
	ldc		{sdw},{cr16}
	ldc		{sdl},{cr24a}

	ldctx		{abs16s},{abs24}

	ldipl		{uimm3}

	{max;min}.b	{simm8},{sdb}
	{max;min}.w	{simm16},{sdw}
	{max;min}.b	{sdb},{sdb}
	{max;min}.w	{sdw},{sdw}

	mov.b{g}	{imm8},{isdb}
	mov.w{g}	{imm16},{isdw}
	mov.l{g}	{imm32},{isdl}
	mov.b:q		{simm4},{isdb}
	mov.w:q		{simm4},{isdw}
	mov.b:s		{imm8},{idst2b}
	mov.w:s		{imm16},{idst2w}
	mov.w:s		{imm16},{a0;a1}
	mov.l:s		{imm24},{a0;a1}
	mov.b:z		#0,{idst2b}
	mov.w:z		#0,{idst2w}
	mov.b{g}	{isdb},{isdb}
	mov.w{g}	{isdw},{isdw}
	mov.l{g}	{isdl},{isdl}
	mov.b:s		{iaddr2},r0l
	mov.w:s		{iaddr2},r0
	mov.b:s		{idst2b},r1l
	mov.w:s		{idst2w},r1
	mov.b:s		r0l,{iaddr2}
	mov.w:s		r0,{iaddr2}
	mov.l:s		{iaddr2},{a0;a1}
	mov.b{g}	{sdsp8}[sp],{sdb}
	mov.w{g}	{sdsp8}[sp],{sdw}
	mov.b{g}	{sdb},{sdsp8}[sp]
	mov.w{g}	{sdw},{sdsp8}[sp]

	mova		{{daddrs};{abs16};{abs24}},{r2r0;r3r1;a0;a1}

	mov{ll;lh;hl;hh}	r0l,{r0l;r0h;r1l;r1h;{addrs}}
	mov{ll;lh;hl;hh}	{r0l;r0h;r1l;r1h;{addrs}},r0l

	movx		{simm8},{isdl}

	mul.b		{simm8},{isdb}
	mul.w		{simm16},{isdw}
	mul.b		{isdb},{isdb}
	mul.w		{isdw},{isdw}

	mulex		{r3;a0;a1;{iaddrs};[{iaddrs}]}

	mulu.b		{uimm8},{isdb}
	mulu.w		{uimm16},{isdw}
	mulu.b		{isdb},{isdb}
	mulu.w		{isdw},{isdw}

	neg.b		{isdb}
	neg.w		{isdw}

	nop

	not.b		{isdb}
	not.w		{isdw}

	or.b{g}		{imm8},{isdb}
	or.w{g}		{imm16},{isdw}
	or.b:s		{imm8},{idst2b}
	or.w:s		{imm16},{idst2w}
	or.b{g}		{isdb},{isdb}
	or.w{g}		{isdw},{isdw}

	pop.b		{isdb}
	pop.w		{isdw}
	popc		{cr16}
	popc		{cr24}
	popm		{;fb,}{;sb,}{;a1,}{;a0,}{;r3,}{;r2,}{;r1,}r0
	popm		{;fb,}sb

	push.b		{imm8}
	push.w		{imm16}
	push.b		{isdb}
	push.w		{isdw}
	push.l		{imm32}
	push.l		{isdl}
	pusha		{daddrs}
	pushc		{cr16}
	pushc		{cr24}
	pushm		{;fb,}{;sb,}{;a1,}{;a0,}{;r3,}{;r2,}{;r1,}r0
	pushm		{;fb,}sb

	reit

	rmpa.{b;w}

	rolc.b		{isdb}
	rolc.w		{isdw}
	rorc.b		{isdb}
	rorc.w		{isdw}

	rot.b		{nimm4},{isdb}
	rot.w		{nimm4},{isdw}
	rot.b		r1h,{isdb}
	rot.w		r1h,{isdw}

	rts

	sbb.b		{imm8},{sdb}
	sbb.w		{imm16},{sdw}
	sbb.b		{sdb},{sdb}
	sbb.w		{sdw},{sdw}

	sbjnz.b		{simm4i},{sdb},{label8}
	sbjnz.w		{simm4i},{sdw},{label8}

	sc{cond}	{sdw}

	scmpu.{b;w}

	sha.b		{nimm4},{isdb}
	sha.w		{nimm4},{isdw}
	sha.l		{nimm4},{isdl}
	sha.b		r1h,{isdb}
	sha.w		r1h,{isdw}
	sha.l		r1h,{isdl}

	shl.b		{nimm4},{isdb}
	shl.w		{nimm4},{isdw}
	shl.l		{nimm4},{isdl}
	shl.b		r1h,{isdb}
	shl.w		r1h,{isdw}
	shl.l		r1h,{isdl}

	sin.{b;w}
	smovb.{b;w}
	smovf.{b;w}
	smovu.{b;w}
	sout.{b;w}
	sstr.{b;w}

	stc		{cr24a},{sdl}
	stc		{cr16},{sdw}

	stctx		{abs16s},{abs24}

	stnz.b		{imm8},{isdb}
	stnz.w		{imm16},{isdw}
	stz.b		{imm8},{isdb}
	stz.w		{imm16},{isdw}
	stzx.b		{imm8},{imm8},{isdb}
	stzx.w		{imm16},{imm16},{isdw}

	sub.b{g}	{imm8},{isdb}
	sub.w{g}	{imm16},{isdw}
	sub.l{g}	{imm32},{isdl}
	sub.b:s		{imm8},{idst2b}
	sub.w:s		{imm16},{idst2w}
	sub.b{g}	{isdb},{isdb}
	sub.w{g}	{isdw},{isdw}
	sub.l{g}	{isdl},{isdl}

	subx		{simm8},{isdl}
	subx		{isdb},{isdl}

	tst.b{g}	{imm8},{sdb}
	tst.w{g}	{imm16},{sdw}
	tst.b:s		{imm8},{dst2b}
	tst.w:s		{imm16},{dst2w}
	tst.b{g}	{sdb},{sdb}
	tst.w{g}	{sdw},{sdw}

	und

	wait

	xchg.b		{r0l;r1l;r0h;r1h;a0;a1},{isdb}
	xchg.w		{r0;r1;r2;r3;a0;a1},{isdw}

	xor.b		{imm8},{isdb}
	xor.w		{imm16},{isdw}
	xor.b		{isdb},{isdb}
	xor.w		{isdw},{isdw}
